#!/bin/bash
#

# buildGCC.sh <product_dir> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

make_tarball()
{
    echo ""
    set -x
    cd ${product_dir}
    tarname=${package}-${pkgdotver}-${OS}-${plat}.tar.bz2
    tar cjf ${tarname} \
       ${package}/${pkgver}.version/${flvr}_ \
       ${package}/${pkgver}/ups \
       ${package}/${pkgver}/${flvr}
    cd ${thisdir}
    set +x
    exit 0
}

make_source_tarball()
{
    set -x
    cd ${product_dir}
    tar cjf ${package}-${pkgdotver}-source.tar.bz2 \
       ${package}/${pkgver}/*.sh \
       ${package}/${pkgver}/ups \
       ${package}/${pkgver}/patch \
       ${package}/${pkgver}/tar
    cd ${thisdir}
    set +x
}

check_with_lsb_release ()
{
   plat=`uname -p`
   if [ `lsb_release -d | cut -f2 | cut  -f1 -d" "` = "Ubuntu" ]
   then
      OS1=ub
      OSnum=`lsb_release -r | cut -f2 | cut -f1 -d"."`
      OS=${OS1}${OSnum}
      plat=`uname -m`
   elif [ ${OS1} = "Linux" ]
   then
      OSnum=`lsb_release -r | cut -f2 | cut -f1 -d"."`
      OS=${OS1}${OSnum}
      # Scientific Linux - slf should work
      if [ `lsb_release -d | cut  -f3 -d" "` = "SL" ]
      then
	 OS=slf${OSnum}
      # Scientific Linux Fermi
      elif [ `lsb_release -d | cut  -f3 -d" "` = "SLF" ]
      then
	 OS=slf${OSnum}
      #
      elif [ `lsb_release -i | cut -f2` = "ScientificFermi" ]
      then
	 OS=slf${OSnum}
      # pretend that SL6 is the same as SLF6
      elif [ `lsb_release -i | cut -f2` = "Scientific" ]
      then
	 OS=slf${OSnum}
      # pretend that CentOS is SLF
      elif [ `lsb_release -i | cut -f2` = "CentOS" ]
      then
	 OS=slf${OSnum}
      # pretend that RedHatEnterpriseServer is SLF
      elif [ `lsb_release -i | cut -f2` = "RedHatEnterpriseServer" ]
      then
	 OS=slf${OSnum}
      # Scientific Linux CERN
      elif [ `lsb_release -d | cut  -f4 -d" "` = "SLC" ]
      then
	 OS=slc${OSnum}
      elif [ `lsb_release -d | cut  -f4 -d" "` = "LTS" ]
      then
	 OS=slf${OSnum}
      # unrecognized - pretend that this is SLF
      else
	 OS=slf${OSnum}
      fi
   fi
}

default_names ()
{
      OSnum1=`uname -r | cut -f1 -d"."`
      OSnum2=`uname -r | cut -f2 -d"."`
      OS=${OS1}${OSnum1}${OSnum2}
      plat=`uname -p`
}

check_linux ()
{
   plat=`uname -p`
   if [ -e /etc/system-release-cpe ]
   then
      OSnum=`cat /etc/system-release-cpe | cut -f5 -d":" |  cut -f1 -d"."`
      if [ `cat /etc/redhat-release | cut  -f4 -d" "` = "SLC" ]
      then
         OS=slc${OSnum}
      else
         OS=slf${OSnum}
      fi
   elif [ -e /etc/redhat-release ]
   then
      if [ `cat /etc/redhat-release | cut  -f4 -d" "` = "SLC" ]
      then
         OS=slc${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f6 -d" " |  cut -f1 -d"."`
      elif [ `cat /etc/redhat-release | cut  -f3 -d" "` = "Fermi" ]
      then
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      elif [ `cat /etc/redhat-release | cut  -f3 -d" "` = "SLF" ]
      then
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      else
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      fi
   else
      default_names
   fi
}

get_platform_name()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
	plat=`uname -m`
	OSnum=`uname -r | cut -f1 -d"."`
	#OS=${OS1}${OSnum}
	mactype=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | wc -w`
	if [ ${mactype} -gt 0 ]
	then
	   macos=`system_profiler  -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 1,2`
	   macos1=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 1`
	   macos2=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 2`
	else 
	   macos=`system_profiler  -xml -detailLevel mini SPSoftwareDataType | grep "OS X" | cut -d ' ' -f 3 | cut -d . -f 1,2`
	   macos1=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep "OS X" | cut -d ' ' -f 3 | cut -d . -f 1`
	   macos2=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep "OS X" | cut -d ' ' -f 3 | cut -d . -f 2`
	fi
	OS=osx${macos1}${macos2}
    elif [ "${flvr0}" = "Linuxppc" ]
    then
       OS=slf5
       plat=`uname -p`
    elif [ -e /usr/bin/lsb_release ]
    then
       check_with_lsb_release
    elif [ ${OS1} = "Linux" ]
    then
       check_linux
    else
       default_names
    fi

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}
    pkgblddir=${product_dir}/${package}/${pkgver}/build/${flvr}
    tardir=${product_dir}/${package}/${pkgver}/tar
    patchdir=${product_dir}/${package}/${pkgver}/patch
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr}"
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table
    else 
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table
    fi
}

unpack_and_patch()
{
    # everything at once

    mkdir -p ${pkgblddir}/src
    # unpack
    for f in ${tardir}/* 
    do 
	tarf=`basename ${f}`
	pf1=`basename ${tarf} .gz`
	pf2=`basename ${pf1} .bz2`
	pf=`basename ${pf2} .tar`
	if [ -e ${pkgblddir}/src/${pf} ]
	then
	   echo "${pkgblddir}/src/${pf} already exists"
	else
	   echo "untar and patch ${tarf}"
	   set -x
           cd ${pkgblddir}/src
	   tar xf $f
	   if [ -f ${patchdir}/${pf}.patch ]
	   then
              cd ${pkgblddir}/src/${pf} && patch -b -p0 < ${patchdir}/${pf}.patch
	      echo "patch ${pf}"
	   fi
	   set +x
	fi
	
    done
    flvr1=`ups flavor -1`
    if [ ${flvr1} = "Darwin" ]
    then
	set -x
	cd ${pkgblddir}/src
	perl -pi.bak -e 's/uname -p/uname -m/go;' */config.guess */configfsf.guess
	set +x
    fi
}


# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
maketar=${2}
sourcetar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=gcc
pkgver=v4_8_1
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`

get_this_dir

if [ "${sourcetar}" = "source" ]
then
  make_source_tarball
fi

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   make_tarball
fi

echo "building ${package} for ${OS}-${plat} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------


mkdir -p ${pkgblddir}
if [ ! -d ${pkgblddir} ]
then
   echo "ERROR: failed to create ${pkgblddir}"
   echo "       Something is very wrong"
   exit 1
fi

unpack_and_patch

# set the build environment
# set CC and CXX
export CC=gcc
export CXX=g++
# remove /usr/local/bin from path
PATH=`dropit /usr/local/bin`
# make sure libraries are in the path
echo "COMPILER_PATH   $COMPILER_PATH"
export PATH=${pkgdir}/bin:${PATH}
export COMPILER_PATH=${pkgdir}/bin
if [ ${OS1} = "Darwin" ]
then
  echo "DYLD_LIBRARY_PATH $DYLD_LIBRARY_PATH"
  export DYLD_LIBRARY_PATH=${pkgdir}/lib64:${pkgdir}/lib
  shext=dylib
else
  echo "LD_LIBRARY_PATH $LD_LIBRARY_PATH"
  export LD_LIBRARY_PATH=${pkgdir}/lib64:${pkgdir}/lib
  shext=so
fi
# need proper environment
echo ""
echo "CC $CC CXX $CXX "
echo "LD_LIBRARY_PATH $LD_LIBRARY_PATH"
echo "DYLD_LIBRARY_PATH $DYLD_LIBRARY_PATH"
echo "COMPILER_PATH   $COMPILER_PATH"

echo ""
echo "------------------------------------------------------------------"
# build gmp
if [ -e ${pkgdir}/lib/libgmp.${shext} ] && [ -e ${pkgdir}/lib/libgmpxx.${shext} ] \
   && [ -e ${pkgdir}/include/gmp.h ] && [ -e ${pkgdir}/include/gmpxx.h ]
then
    echo "gmp has already been built"
else
    set -x
    cd ${pkgblddir}/src/gmp-*
    cp configfsf.guess config.guess
    CPPFLAGS=-fexceptions ./configure --prefix=${pkgdir} --enable-cxx 
    make all check install
    set +x
fi
if [ ! -e ${pkgdir}/lib/libgmp.${shext} ] || [ ! -e ${pkgdir}/lib/libgmpxx.${shext} ] \
   || [ ! -e ${pkgdir}/include/gmp.h ] || [ ! -e ${pkgdir}/include/gmpxx.h ]
then
   echo "ERROR: the gmp build has failed"
   echo "       Something is very wrong"
   exit 1
fi

echo ""
echo "------------------------------------------------------------------"
# build ppl
if [ -e ${pkgdir}/lib/libppl.${shext} ] && [ -e ${pkgdir}/lib/libppl_c.${shext} ] \
   && [ -e ${pkgdir}/bin/ppl-config ] \
   && [ -e ${pkgdir}/include/ppl.hh ] && [ -e ${pkgdir}/include/ppl_c.h ]
then
    echo "ppl has already been built"
else
    set -x
    cd ${pkgblddir}/src/ppl-*
    CXXFLAGS="-fno-access-control" CPPFLAGS=-I${pkgdir}/include LDFLAGS=-L${pkgdir}/lib ./configure --prefix=${pkgdir}  --without-java --with-gmp=${pkgdir} 
    make all install
    set +x
fi
if [ ! -e ${pkgdir}/lib/libppl.${shext} ] || [ ! -e ${pkgdir}/lib/libppl_c.${shext} ] \
   || [ ! -e ${pkgdir}/bin/ppl-config ] \
   || [ ! -e ${pkgdir}/include/ppl.hh ] || [ ! -e ${pkgdir}/include/ppl_c.h ]
then
   echo "ERROR: the ppl build has failed"
   echo "       Something is very wrong"
   exit 1
fi

echo ""
echo "------------------------------------------------------------------"
# build mpfr
if [ -e ${pkgdir}/lib/libmpfr.${shext} ] \
   && [ -e ${pkgdir}/include/mpfr.h ] && [ -e ${pkgdir}/include/mpf2mpfr.h ]
then
    echo "mpfr has already been built"
else
    set -x
    cd ${pkgblddir}/src/mpfr-*
    ./configure --prefix=${pkgdir} --with-gmp=${pkgdir} 
    make all
    make install
    set +x
fi
if [ ! -e ${pkgdir}/lib/libmpfr.${shext} ] \
   || [ ! -e ${pkgdir}/include/mpfr.h ] || [ ! -e ${pkgdir}/include/mpf2mpfr.h ]
then
   echo "ERROR: the mpfr build has failed"
   echo "       Something is very wrong"
   exit 1
fi

echo ""
echo "------------------------------------------------------------------"
# build mpc
if [ -e ${pkgdir}/lib/libmpc.${shext} ] \
   && [ -e ${pkgdir}/include/mpc.h ] 
then
    echo "mpc has already been built"
else
    set -x
    cd ${pkgblddir}/src/mpc-*
    ./configure --prefix=${pkgdir} --with-mpfr=${pkgdir} --with-gmp=${pkgdir} 
    make all install
    set +x
fi
if [ ! -e ${pkgdir}/lib/libmpc.${shext} ] \
   || [ ! -e ${pkgdir}/include/mpc.h ] 
then
   echo "ERROR: the mpc build has failed"
   echo "       Something is very wrong"
   exit 1
fi

echo ""
echo "------------------------------------------------------------------"
# build cloog
if [ -e ${pkgdir}/lib/libcloog-isl.${shext} ] && [ -e ${pkgdir}/lib/libisl.${shext} ] \
   && [ -e ${pkgdir}/bin/cloog ] \
   && [ -d ${pkgdir}/include/cloog ] && [ -d ${pkgdir}/include/isl ]
then
    echo "cloog has already been built"
else
    set -x
    cd ${pkgblddir}/src/cloog-*
    ./configure --prefix=${pkgdir} -with-ppl=${pkgdir} --with-gmp-prefix=${pkgdir} 
    make all check install
    set +x
fi
if [ ! -e ${pkgdir}/lib/libcloog-isl.${shext} ] || [ ! -e ${pkgdir}/lib/libisl.${shext} ] \
   || [ ! -e ${pkgdir}/bin/cloog ] \
   || [ ! -d ${pkgdir}/include/cloog ] || [ ! -d ${pkgdir}/include/isl ]
then
   echo "ERROR: the cloog build has failed"
   echo "       Something is very wrong"
   exit 1
fi

echo ""
echo "------------------------------------------------------------------"
# build gcc
if [ -e ${pkgdir}/lib/libstdc++.${shext} ] && [ -e ${pkgdir}/lib/libgfortran.${shext} ] \
   && [ -e ${pkgdir}/bin/gcc ] && [ -e ${pkgdir}/bin/g++ ] && [ -e ${pkgdir}/bin/gfortran ] \
   && [ -d ${pkgdir}/include/c++ ]
then
    echo "gcc has already been built"
else
    set -x
    cd ${pkgblddir}/src/gcc-*
    export CXXFLAGS="-fPIC"
    export CFLAGS="-fPIC"
    enable_watchdog=yes ./configure --prefix=${pkgdir} \
                                    --with-mpfr=${pkgdir} \
                                    --with-mpc=${pkgdir} \
				    --with-gmp=${pkgdir} \
				    --with-ppl=${pkgdir} \
				    --with-cloog=${pkgdir} \
				    --enable-cloog-backend=isl  \
				    --with-gnu-as --with-gnu-ld \
				    --disable-libgcj --enable-threads=posix \
				    --enable-languages=c,c++,objc,fortran
    make -j20 bootstrap
    make install
    set +x
fi
if [ ! -e ${pkgdir}/lib/libstdc++.${shext} ] || [ ! -e ${pkgdir}/lib/libgfortran.${shext} ] \
   || [ ! -e ${pkgdir}/bin/gcc ] || [ ! -e ${pkgdir}/bin/g++ ] || [ ! -e ${pkgdir}/bin/gfortran ] \
   || [ ! -d ${pkgdir}/include/c++ ]
then
   echo "ERROR: the gcc build has failed"
   echo "       Something is very wrong"
   exit 1
fi

echo ""
echo "------------------------------------------------------------------"
# build gdb
if [ -e ${pkgdir}/bin/gdb ] 
then
    echo "gdb has already been built"
else
    set -x
    cd ${pkgblddir}/src/gdb-*
    export CXXFLAGS="-fPIC"
    export CFLAGS="-fPIC"
    enable_watchdog=yes ./configure --prefix=${pkgdir} \
                                    --with-mpfr=${pkgdir} \
                                    --with-mpc=${pkgdir} \
				    --with-gmp=${pkgdir} \
				    --with-ppl=${pkgdir} \
				    --with-cloog=${pkgdir} \
				    --with-python=no \
				    --with-gnu-as --with-gnu-ld \
				    --disable-libgcj --enable-threads=posix \
				    --enable-languages=c,c++,objc,fortran
    make all
    make install
    set +x
fi
if [ ! -e ${pkgdir}/bin/gdb ]
then
   echo "ERROR: the gdb build has failed"
   echo "       Something is very wrong"
   exit 1
fi


if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

declare_product

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver}
echo "${package} is installed at ${GCC_FQ_DIR}"


# this must be last because it will exit
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   make_tarball
fi

exit 0
